// CategorySelector.as
// John Maloney, August 2009
//
// CategorySelector is a UI widget that holds set of CategorySelectorItems
// and support changing the selected category. When the category is changed,
// the block palette is filled with the blocks for the selected category.

package UI {
	import flash.display.Sprite;
	import Blocks.Block;

public class CategorySelector extends Sprite {

	public var selectedCategory:int = 0;
	public var blockPalette:BlockPalette;
	public var commandSpecs:Array;
	public var categorySpecs:Array;

	public function CategorySelector(commandSpecs:Array, categorySpecs:Array) {
		this.commandSpecs = commandSpecs;
		this.categorySpecs = categorySpecs;
		setWidthHeightColor(250, 85, 0x808080);
	}

	public function initCategories(catList:Array):void {
		var x:int = 30, y:int = 10, i:int, maxW:int = 0;
		var entry:Array;
		var item:CategorySelectorItem;
		for (i = 0; i < catList.length; i++) {
			if (i == 3) {
				x += maxW + 55;
				y = 10;
			}
			entry = catList[i];
			item = new CategorySelectorItem(entry[0], entry[1]);
			item.x = x;
			item.y = y;
			addChild(item);
			y += item.height + 2;
			maxW = Math.max(maxW, item.width);
		}
		select(1);
	}

	public function setWidthHeightColor(w:int, h:int, c:int):void {
		graphics.clear();
		graphics.beginFill(c);
		graphics.drawRect(0, 0, w, h);
	}

	public function select(id:int):void {
		var i:int;
		for (i = 0; i < numChildren; i++) {
			if (getChildAt(i) is CategorySelectorItem) {
				var item:CategorySelectorItem = CategorySelectorItem(getChildAt(i));
				item.setSelected(item.id == id);
			}
		}
		selectedCategory = id;
		if (blockPalette != null) showBlocksForCategory(id);
	}

	public function showBlocksForCategory(selectedCategory:int):void {
		var b:Block, i:int, y:int = 15;
		var catColor:int = categoryColor(selectedCategory);
		while (blockPalette.numChildren > 0) blockPalette.removeChildAt(0); // remove all children
		for (i = 0; i < commandSpecs.length; i++) {
			var item:Array = commandSpecs[i];
			var itemCat:int = item[2];
			if (itemCat == selectedCategory) {
				b = new Block(item[0], item[1], catColor, item[3]);
				if (item.length > 4) b.setDefaultArgs(item.slice(4));
				b.x = 15;
				b.y = y;
				blockPalette.addChild(b);
				y += b.height + 5;
			}
		}
	}

	private function categoryColor(categoryID:int):int {
		var i:int = (categoryID < categorySpecs.length) ? categoryID : 0;
		return categorySpecs[i][2];
	}

}}
