// CmdSpecs.as
// John Maloney, August 2009
//
// This file defines the command blocks and categories for a given blocks language.
// To add a new command:
//		a. define a unique opcode constant for the command (e.g. PLUS = 30)
//		b. create an entry for the new command in the commands array
//		c. add a case for the new command to the interpreter

package {

public class CmdSpecs {

	static public var categories:Array = [
	  // id   category name	  color
		[0,  "no category",	0xD42828],
		[1,  "Control",		0xE6A822],
		[2,  "Graphics",	0x8F56E3],
		[3,  "Sound",		0xCF4AD9],
		[4,  "Operators",	0x62C213],
		[5,  "Sensing",		0x0494DC],
		[6,  "Variables",	0xF3761D],
	];

	static public var commands:Array = [
		// block specification			type cat opcode			default args (optional)
		["hat test -- does nothing",	"h", 1, NOOP],
		["forever",						"cf",1, FOREVER],
		["repeat %n",					"c", 1, REPEAT],
		["if %b",						"c", 1, IF],
		["wait %n secs",				" ", 1, WAIT,			1],
		["stop script",					"f", 1, STOP],
		["stop all",					"f", 1, STOP_ALL],

		["clear graphics",				" ", 2, CLEAR_GRAPHICS],
		["set color r %n g %n b %n",	" ", 2, SET_RGB,		0, 0, 255],
		["set hue to %n",				" ", 2, SET_HUE,		0],
		["change hue by %n",			" ", 2, CHANGE_HUE,		30],
		["rectangle %n %n w %n h %n",	" ", 2, RECTANGLE,		10, 10, 100, 100],
		["ellipse %n %n w %n h %n",		" ", 2, ELLIPSE,		10, 10, 100, 100],
		["clear text",					" ", 2, CLEAR_TEXT],
		["print %s",					" ", 2, PRINT,			"hello"],
		["print frame rate",			" ", 2, PRINT_FRAMERATE],

		["pop",							" ", 3, PLAY_POP],

		["%n + %n",						"r", 4, PLUS],
		["%n - %n",						"r", 4, MINUS],
		["%n * %n",						"r", 4, TIMES],
		["%n / %n",						"r", 4, DIVIDE],
		["%n % %n",						"r", 4, MOD],
		["%n < %n",						"b", 4, LESS],
		["%n = %n",						"b", 4, EQUAL],
		["%n > %n",						"b", 4, GREATER],
		["%b and %b",					"b", 4, AND],
		["%b or %b",					"b", 4, OR],
		["not %b",						"b", 4, NOT],
		["pick random %n to %n",		"r", 4, RANDOM,			1, 10],
		["round %n",					"r", 4, ROUND, 			1.5],
		["join %s%s",					"r", 4, JOIN, 			"Hello, ", "World!"],

		["mouse down",					"b", 5, MOUSE_DOWN],
		["mouse x",						"r", 5, MOUSE_X],
		["mouse y",						"r", 5, MOUSE_Y],
		["timer",						"r", 5, TIMER],
		["clear timer",					" ", 5, CLEAR_TIMER],

		["var %s",						"r", 6, GET_VAR,		"v"],
		["set %s to %n",				" ", 6, SET_VAR,		"v", 0],
		["change %s by %n",				" ", 6, CHANGE_VAR,		"v", 1],
	];

	// control op codes
	static public var NOOP:int = 0;
	static public var FOREVER:int = 1;
	static public var REPEAT:int = 2;
	static public var IF:int = 3;
	static public var WAIT:int = 4;
	static public var STOP:int = 5;
	static public var STOP_ALL:int = 6;

	// math/string op codes
	static public var PLUS:int = 20;
	static public var MINUS:int = 21;
	static public var TIMES:int = 22;
	static public var DIVIDE:int = 23;
	static public var MOD:int = 24;
	static public var LESS:int = 25;
	static public var EQUAL:int = 26;
	static public var GREATER:int = 27;
	static public var AND:int = 28;
	static public var OR:int = 29;
	static public var NOT:int = 30;
	static public var RANDOM:int = 31;
	static public var ROUND:int = 32;
	static public var JOIN:int = 33;

	// variables op codes
	static public var GET_VAR:int = 40;
	static public var SET_VAR:int = 41;
	static public var CHANGE_VAR:int = 42;

	// sensing op codes
	static public var MOUSE_DOWN:int = 50;
	static public var MOUSE_X:int = 51;
	static public var MOUSE_Y:int = 52;
	static public var TIMER:int = 53;
	static public var CLEAR_TIMER:int = 54;

	// graphics op codes
	static public var CLEAR_GRAPHICS:int = 60;
	static public var SET_RGB:int = 61;
	static public var SET_HUE:int = 62;
	static public var CHANGE_HUE:int = 63;
	static public var RECTANGLE:int = 64;
	static public var ELLIPSE:int = 65;
	static public var PRINT:int = 66;
	static public var PRINT_FRAMERATE:int = 67;
	static public var CLEAR_TEXT:int = 68;

	// sound op codes
	static public var PLAY_POP:int = 70;

}}
