// Thread.as
// John Maloney, August 2009
//
// Thread is an internal data structure used by Interpreter.as. It holds the
// state of a thread so it can continue from where it left off, and it supports
// a pushdown stack of states to support control structures such as loops.

package {
	import Blocks.Block;
	
public class Thread {

	public var topBlock:Block;
	public var stack:Array = [];

	// the following state is pushed and popped when running substacks
	public var block:Block;
	public var isLoop:Boolean;
	public var firstTime:Boolean;	// used by certain control structures
	public var tmp:int;				// used by repeat and wait

	public function Thread(b:Block) {
		topBlock = b;
		initForBlock(b);
	}

	public function initForBlock(b:Block):void {
		block = b;
		isLoop = false;
		firstTime = true;
		tmp = 0;
	}

	public function pushState():void {
		stack.push([
			block, isLoop, firstTime, tmp
		]);
	}

	public function popState():void {
		if (stack.length == 0) return;
		var a:Array = stack.pop();
		block		= a[0];
		isLoop		= a[1];
		firstTime	= a[2];
		tmp			= a[3];
	}

}}
